#include "delay.h"
#include "sys.h"
#include "FreeRTOS.h"
#include "task.h"
#include "ch395.h"
#include "ch395inc.h"
#include "CH395_network_operation.h"
#include "CH395_interface_operation.h"
#include "MCU_communication_interface.h"
#include "transport.h"
#include "MQTTPacket.h"
#include "semphr.h"
#include "https.H"
#include "html.h"


#define START_TASK_PRIO		1
#define START_STK_SIZE 		128  
TaskHandle_t StartTask_Handler;
void start_task(void *pvParameters);

#define HTTP_TASK_PRIO		3
#define HTTP_STK_SIZE 		1024  
TaskHandle_t HTTPTask_Handler;
void http_task(void *pvParameters);

#define MQTT_TASK_PRIO		2
#define MQTT_STK_SIZE 		512  
TaskHandle_t MQTTTask_Handler;
void mqtt_task(void *pvParameters);

#define INT_TASK_PRIO		 4
#define INT_STK_SIZE 		 512  
TaskHandle_t INTTask_Handler;
void ch395int_task(void *pvParameters);




/**********************************************
***                  RTOS
**********************************************/
SemaphoreHandle_t BinarySemaphore_MQTT;	
SemaphoreHandle_t BinarySemaphore_WEB;	
BaseType_t xHigherPriorityTaskWoken_MQTT;
BaseType_t xHigherPriorityTaskWoken_WEB;
BaseType_t err=pdFALSE;
/**********************************************
***                 ETH_NET
**********************************************/
//WEB_SERVER
uint8_t http_sock;
uint8_t web_flag;		
uint8_t Http_SocketRecvBuf[1024]; 
st_http_request *http_request; 
uint8_t  flag;		   		  									                                             
uint16_t http_port = 80;
//MQTT
uint8_t mqtt_ip[4] = {192,168,1,100};
uint8_t mqtt_sock = 0;
uint16_t MQTT_PORT = 61613;
uint16_t srcport = 2000;
uint8_t con_flag=0;										                      
uint8_t pub_flag=1;											                   
uint8_t sub_flag=0;											                  
uint8_t tout_flag=0;											                   
uint16_t packetid=0;											                 
uint8_t sub_buff[512];
char *username  = "admin";							                             
char *password  = "password";								                         
char *sub_topic = "news";								                          
char *pub_topic = "topic";	
unsigned char pub_message[] = "l am publishing";

//CH395 NET INFORMATION
uint8_t local_ip[4] = {192,168,1,40};                  
uint8_t gateway_ip[4] = {192,168,1,1};                
uint8_t mask_ip[4] = {255,255,255,0};                   

struct   _SOCK_INF sockinf[6];                                
struct   _CH395_SYS  ch395inf;                                                            


/**********************************************************************************
* Function Name  : TIM3_Int_Init
* Description    : 设置TIM3定时器的中断参数
* Input          : arr---相对时间
* Output         : psc---时间基准
* Return         : None
**********************************************************************************/
void TIM3_Int_Init(uint16_t arr,uint16_t psc)
{
	TIM_TimeBaseInitTypeDef TIM_TimeBaseInitStructure;
	NVIC_InitTypeDef NVIC_InitStructure;
	
	RCC_APB1PeriphClockCmd(RCC_APB1Periph_TIM3,ENABLE);  
	
  TIM_TimeBaseInitStructure.TIM_Period = arr; 	
	TIM_TimeBaseInitStructure.TIM_Prescaler=psc;  
	TIM_TimeBaseInitStructure.TIM_CounterMode=TIM_CounterMode_Up; 
	TIM_TimeBaseInitStructure.TIM_ClockDivision=TIM_CKD_DIV1; 
	
	TIM_TimeBaseInit(TIM3,&TIM_TimeBaseInitStructure);
	
	TIM_ITConfig(TIM3,TIM_IT_Update,ENABLE); 
	TIM_Cmd(TIM3,ENABLE); 
	
	NVIC_InitStructure.NVIC_IRQChannel=TIM3_IRQn; 
	NVIC_InitStructure.NVIC_IRQChannelPreemptionPriority=0x05;
	NVIC_InitStructure.NVIC_IRQChannelSubPriority=0x00; 
	NVIC_InitStructure.NVIC_IRQChannelCmd=ENABLE;
	NVIC_Init(&NVIC_InitStructure);
}
/**********************************************************************************
* Function Name  : TIM3_IRQHandler
* Description    : MQTT定时发心跳包
* Input          : None
* Output         : None
* Return         : None
**********************************************************************************/
void TIM3_IRQHandler(void)
{
	static uint8_t num;
	unsigned char* payload = pub_message;
	
	if(TIM_GetITStatus(TIM3,TIM_IT_Update)==SET) 
	{
		if(sub_flag && (num==10))
		{
			MQTT_Pingreq(mqtt_sock);                //MQTT心跳包：60s没有信息交换，MQTT会断线
			num = 0;
		}
		if(con_flag)
		{
			MQTT_Publish(mqtt_sock, pub_topic, payload);     //向某个主题发布消息，可自行定义时间和方式
		}
		num++;
	}
	TIM_ClearITPendingBit(TIM3,TIM_IT_Update);  
}

/**********************************************************************************
* Function Name  : CH395GlobalInterrupt
* Description    : CH395
* Input          : None
* Output         : None
* Return         : None
**********************************************************************************/
void CH395GlobalInterrupt(void)
{
	uint16_t  init_status;
	
	
	init_status = ch395_cmd_get_glob_int_status_all();
	if(init_status & GINT_STAT_UNREACH)                              
	{                               
	}
	if(init_status & GINT_STAT_IP_CONFLI)                           
	{
	}
	if(init_status & GINT_STAT_PHY_CHANGE)                           
	{
		printf("Init status : GINT_STAT_PHY_CHANGE\r\n");
		transport_open(mqtt_sock,mqtt_ip,srcport++);
		ch395_socket_init();
		socket_open();
		TIM3_Int_Init(50000-1,7200-1);
	}
	if(init_status & GINT_STAT_SOCK0)
	{
		if(BinarySemaphore_MQTT!=NULL)
	  {
			xSemaphoreGiveFromISR(BinarySemaphore_MQTT,&xHigherPriorityTaskWoken_MQTT);	
//			portYIELD_FROM_ISR(xHigherPriorityTaskWoken_MQTT_SUB);
			vTaskSuspend(NULL);
		}
	}
	if(init_status & GINT_STAT_SOCK1) 
    http_sock = 1;
	if(init_status & GINT_STAT_SOCK2) 
		http_sock = 2;
	if(init_status & GINT_STAT_SOCK3) 
		http_sock = 3;
	if(init_status & GINT_STAT_SOCK4)
		http_sock = 4;
	if(((init_status & GINT_STAT_SOCK1)||(init_status & GINT_STAT_SOCK2)||(init_status & GINT_STAT_SOCK3)||(init_status & GINT_STAT_SOCK4)) &&(BinarySemaphore_WEB!=NULL))
	  {
			xSemaphoreGiveFromISR(BinarySemaphore_WEB,&xHigherPriorityTaskWoken_WEB);	
//			portYIELD_FROM_ISR(xHigherPriorityTaskWoken_WEB);
			vTaskSuspend(NULL);
		}
}


int main(void)
{
	uint8_t rc;
	NVIC_PriorityGroupConfig(NVIC_PriorityGroup_4);	 
	delay_init();	    				 
	uart_init(921600);					
	
	//ch395 INITIALILIZE
	CH395_SPI_Init();
	ch395_inf_init();
  ch395_cmd_hard_reset();
	delay_ms(500);
  rc=CH395_init();                             
  stop_if_error(rc); 
	Socket_R_S_Buf_Modify();
	http_request = (st_http_request*)Http_SocketRecvBuf;
	
  xTaskCreate((TaskFunction_t )start_task,            
                (const char*    )"start_task",          
                (uint16_t       )START_STK_SIZE,        
                (void*          )NULL,                  
                (UBaseType_t    )START_TASK_PRIO,      
                (TaskHandle_t*  )&StartTask_Handler);               
  vTaskStartScheduler();        
}


void start_task(void *pvParameters)
{
    taskENTER_CRITICAL();           

	  BinarySemaphore_MQTT = xSemaphoreCreateBinary();
	  BinarySemaphore_WEB = xSemaphoreCreateBinary(); 
	
    xTaskCreate((TaskFunction_t )http_task,     	
                (const char*    )"http_task",   	
                (uint16_t       )HTTP_STK_SIZE, 
                (void*          )NULL,				
                (UBaseType_t    )HTTP_TASK_PRIO,	
                (TaskHandle_t*  )&HTTPTask_Handler);   

    xTaskCreate((TaskFunction_t )mqtt_task,     
                (const char*    )"mqtt_task",   
                (uint16_t       )MQTT_STK_SIZE, 
                (void*          )NULL,
                (UBaseType_t    )MQTT_TASK_PRIO,
                (TaskHandle_t*  )&MQTTTask_Handler);         
								
    xTaskCreate((TaskFunction_t )ch395int_task,     
                (const char*    )"int_task",   
                (uint16_t       )INT_STK_SIZE, 
                (void*          )NULL,
                (UBaseType_t    )INT_TASK_PRIO,
                (TaskHandle_t*  )&INTTask_Handler);  
								
    vTaskDelete(StartTask_Handler); 			
    taskEXIT_CRITICAL();           
}


void http_task(void *pvParameters)
{
	uint8_t rc;
	uint16_t init_status;
	uint32_t len;
	
  while(1)
  {
		if(BinarySemaphore_WEB!=NULL)
		{
			err=xSemaphoreTake(BinarySemaphore_WEB,portMAX_DELAY);	
			if(err==pdTRUE)									
			{
				vTaskSuspend(MQTTTask_Handler);	
				init_status = ch395_get_socket_int(http_sock);
				if(init_status == SINT_STAT_SENBUF_FREE)
				{
					if(sockinf[http_sock].SendLen >= sockinf[http_sock].RemLen)
          {
						printf("##\r\n");
						sockinf[http_sock].RemLen = 0;                                         
						rc=ch395_tcp_disconnect(http_sock);
						stop_if_error(rc);
					}
					else
					{
						sockinf[http_sock].pSend += sockinf[http_sock].SendLen;                          
						sockinf[http_sock].RemLen -= sockinf[http_sock].SendLen;                         
						if(sockinf[http_sock].RemLen > 1024)sockinf[http_sock].SendLen = 1024;           
						else sockinf[http_sock].SendLen = sockinf[http_sock].RemLen;   
						ch395_send_data(http_sock,sockinf[http_sock].pSend,sockinf[http_sock].SendLen);     
					}		 
				}
				if(init_status == SINT_STAT_RECV)
				{
					len = ch395_get_recv_length(http_sock);
					ch395_get_recv_data(http_sock,len,Http_SocketRecvBuf);
					flag  |= RECVDATA;
					WebServer(http_sock);
				}
				if(init_status & SINT_STAT_CONNECT)
				{
				}
				if(init_status & SINT_STAT_DISCONNECT); 
				if(init_status & SINT_STAT_TIM_OUT); 				
				
				vTaskResume(MQTTTask_Handler);
				vTaskResume(INTTask_Handler);
			}
		}
  }
}   

void mqtt_task(void *pvParameters)
{
	uint16_t init_status;
	uint32_t len,i;
	unsigned char dup1;
  unsigned char* dup = &dup1;
	unsigned short* packetid;
  int qos1;
	int* qos = &qos1;
	unsigned char retained1;
	unsigned char* retained = &retained1;
	MQTTString topicName1;
	MQTTString* topicName = &topicName1;
	unsigned char payload1;
	unsigned char* payload = &payload1;
	int payloadlen;
	unsigned char *p=payload;
	
  while(1)
  {
		if(BinarySemaphore_MQTT!=NULL)
		{
			err=xSemaphoreTake(BinarySemaphore_MQTT,portMAX_DELAY);	
			if(err==pdTRUE)									
			{
				vTaskSuspend(HTTPTask_Handler);
				init_status = ch395_get_socket_int(mqtt_sock);
				if(init_status == SINT_STAT_RECV)
				{
					len = ch395_get_recv_length(mqtt_sock);
					ch395_get_recv_data(mqtt_sock,len,sub_buff);
					switch(sub_buff[0]>>4)
					{
						case CONNACK:
								con_flag=1;
								MQTT_Subscribe(mqtt_sock,sub_topic);
								break;
						case PUBLISH:
								MQTTDeserialize_publish(dup,qos,retained,packetid,topicName,&payload,&payloadlen,sub_buff,len);
								printf("payloadlen=%d\r\n",(uint16_t)payloadlen);
				        p=payload;
								for(i=0;i<payloadlen;i++)
								{
									printf("%c ",(uint16_t)*p);
									p++;
								}
								printf("\r\n");
								break;
						case SUBACK:
								sub_flag=1;
								break;
						default:
								break;
					}
				}
				if(init_status & SINT_STAT_CONNECT)
				{
					MQTT_Connect(mqtt_sock, username,password);
				}
				if(init_status & SINT_STAT_DISCONNECT); 
				if(init_status & SINT_STAT_TIM_OUT); 		
			}
			vTaskResume(HTTPTask_Handler);
			vTaskResume(INTTask_Handler);
		}
	}
}

void ch395int_task(void *pvParameters)
{
    while(1)
    {
			if(ch395_exti_stat==0)
				CH395GlobalInterrupt();	
    }
}


